//-----------------------------------------------------------------------------
// eqandvol.m
//
// created by Stemybacsi
//-----------------------------------------------------------------------------

// never forget to include std.mi
#include "lib/std.mi"

//defines PI as a constant if it hasn't been already
#ifndef PI
#define PI 3.1415926536
#endif

//declares volume functions
Function UpdateEQSelect(Int sFrame);
Function setEqAnim(int eqValue, Layer eqName);
Function UpdateSlider(int x, int y);

//declares rotate functions
Function rotateKnob(float newvol);
Function KnobInit();
Function KnobRotateDegree(double r);
Function KnobRotate(double r);

//declares global variables
Global AnimatedLayer EQSelect;
Global Layer Volume, EQPreamp, eq0, eq1, eq2, eq3, eq4, eq5, eq6, eq7, eq8, eq9;
Global Map VolumeMap, eqMap;
Global Button EQBandB;
Global Timer SongTickerTimer, ChangeTimer;
Global Text SongTicker;
Global Double KnobR;
Global Boolean Changing;
Global Int Changeable;

//when script is loaded, do this
System.onScriptLoaded() {

	// Get the group that has the objects we want
	Group pcGroup = getScriptGroup();

	// Now that we have the group, get the objects in the group
	Volume = pcGroup.findObject("volumeknob");
	SongTicker = pcGroup.findObject("songticker");
	EQBandB = pcGroup.findObject("EQBandB");
	EQSelect = pcGroup.findObject("layer.eqselect");

	//get Layers
	EQPreamp = pcGroup.findObject("EQPreamp");
	eq0 = pcGroup.findObject("EQBar0");
	eq1 = pcGroup.findObject("EQBar1");
	eq2 = pcGroup.findObject("EQBar2");
	eq3 = pcGroup.findObject("EQBar3");
	eq4 = pcGroup.findObject("EQBar4");
	eq5 = pcGroup.findObject("EQBar5");
	eq6 = pcGroup.findObject("EQBar6");
	eq7 = pcGroup.findObject("EQBar7");
	eq8 = pcGroup.findObject("EQBar8");
	eq9 = pcGroup.findObject("EQBar9");

	//initialize Map
	EQMap = new Map;
	EQMap.loadMap("player.EQBarMap");

	//initialize Band positions
	setEqAnim(128 - System.getEqPreamp(), EQPreamp);
	setEqAnim(128 - System.getEqBand(0), eq0);
	setEqAnim(128 - System.getEqBand(1), eq1);
	setEqAnim(128 - System.getEqBand(2), eq2);
	setEqAnim(128 - System.getEqBand(3), eq3);
	setEqAnim(128 - System.getEqBand(4), eq4);
	setEqAnim(128 - System.getEqBand(5), eq5);
	setEqAnim(128 - System.getEqBand(6), eq6);
	setEqAnim(128 - System.getEqBand(7), eq7);
	setEqAnim(128 - System.getEqBand(8), eq8);
	setEqAnim(128 - System.getEqBand(9), eq9);

	// Initialize our timer
	SongTickerTimer = new Timer;
	SongTickerTimer.setDelay(750);

	// Initialize our timer2
	ChangeTimer = new Timer;
	ChangeTimer.setDelay(3000);

	// Load the gradient map
	volumeMap = new Map;
	volumeMap.loadMap("player.map.volknob");


	//initialize rotation and volume variables and set knob position
	KnobInit();
	Changing = 0;
	Changeable = 11;
	UpdateEQSelect(Changeable);
}

// Clears text area
SongTickerTimer.onTimer() {
	SongTicker.setAlternateText("");
	SongTickerTimer.stop();
}
//Sets the Changeable slider to VOlume
ChangeTimer.onTimer() {
	if (!Changing) {
		Changeable = 11;
		UpdateEQSelect(Changeable);
		ChangeTimer.stop();
	}
}

// Handles EQBand Mouse Events
EQBandB.onLeftButtonDown(int x, int y) {
	if (Changeable >= 10) {
		Changeable = 0;
	}
	else {
		Changeable +=1;
	}
	UpdateEQSelect(Changeable);
}
EQBandB.onLeftButtonUp(int x, int y) {
	ChangeTimer.start();
}
// Handles Volume Mouse Events
Volume.onLeftButtonDown(int x, int y) {
	Changing = 1;
	UpdateSlider(x, y);
}
Volume.onLeftButtonUp(int x, int y) {
	if (Changing) {
		Changing = 0;
		UpdateSlider(x, y);
	}
}
Volume.onMouseMove(int x, int y) {
	if (Changing) {
		UpdateSlider(x, y);
	}
}

// Updates Volume Image and System Position
UpdateSlider(int x, int y) {
	int newValue = VolumeMap.getValue(x - Volume.getLeft(), y - Volume.getTop());
	if (Changeable == 11) {
		System.setVolume(newValue);
	}
	else if (Changeable == 10) {
		newValue = - (128 - newValue);
		System.setEqPreamp(newValue);
	}
	else {
		newValue = - (128 - newValue);
		System.setEqBand(Changeable, newValue);
	}
}

// Updates display and SongTicker on Volume changes
System.onVolumeChanged(int newVol) {
	Float f = getVolume();
	f = f / 255;
	rotateKnob(f);
	Songtickertimer.stop();
	Songtickertimer.start();
	int p = (newVol * 100) / 255;
	Songticker.setAlternateText("Volume: " + System.integerToString(p) + "%");
}
// Updates SongTicker on EQBand changes
System.onEqBandChanged(int band, int newValue) {
	Float f = 128 - newValue;
	rotateKnob(f);
	Songtickertimer.stop();
	Songtickertimer.start();
	int p = (newValue * 100) / 255;
	Songticker.setAlternateText("EQ" + System.integerToString(band) + ": " + System.integerToString(p));
	//Kijelz lltsa
	if (band == 0) {
		setEqAnim(128 - newValue, eq0);
	} else if (band == 1) {
		setEqAnim(128 - newValue, eq1);
	} else if (band == 2) {
		setEqAnim(128 - newValue, eq2);
	} else if (band == 3) {
		setEqAnim(128 - newValue, eq3);
	} else if (band == 4) {
		setEqAnim(128 - newValue, eq4);
	} else if (band == 5) {
		setEqAnim(128 - newValue, eq5);
	} else if (band == 6) {
		setEqAnim(128 - newValue, eq6);
	} else if (band == 7) {
		setEqAnim(128 - newValue, eq7);
	} else if (band == 8) {
		setEqAnim(128 - newValue, eq8);
	} else if (band == 9) {
		setEqAnim(128 - newValue, eq9);
	}
}
// Updates SongTicker on EQPreamp changes
System.onEqPreampChanged(int newValue) {
	Float f = 128 - newValue;
	rotateKnob(f);
	Songtickertimer.stop();
	Songtickertimer.start();
	int p = (newValue * 100) / 255;
	Songticker.setAlternateText("PreAmp:" + System.integerToString(p));
	setEqAnim(128 - newValue, EQPreamp);
}

//Belltja a kijelzt
setEqAnim(int eqValue, Layer eqName) {
	Region r = new Region;
	r.loadFromMap(EQMap, eqValue, 0);
	eqName.setRegion(r);
	delete r;
}

UpdateEQSelect(Int sFrame){
	EQSelect.setStartFrame(sFrame);
     	EQSelect.setEndFrame(sFrame);
     	EQSelect.play();
}

//rotates knob to new orientation
rotateKnob(float newvol){
	//finds out how many degrees to rotate
	newvol = newvol * 360;
	//rotates that many degress, uses negative since we want clockwise rotation
	KnobRotateDegree(-newvol);
}

//initializes layer fx settings
KnobInit() {
	Volume.fx_setGridSize(10,10);
	Volume.fx_setBgFx(0);
	Volume.fx_setWrap(0);
	Volume.fx_setBilinear(1);
	Volume.fx_setRect(0);
	Volume.fx_setClear(0);
	Volume.fx_setLocalized(1);
	Volume.fx_setRealtime(0);
	Volume.fx_setEnabled(1);
}

//converts degrees to radians, then calls KnobRotate
KnobRotateDegree(double r) {
	KnobRotate(r * PI / 180.0);
}

//rotates layer
KnobRotate(double r) {
	KnobR = r;
	//notifies that a change has been made, therefor update display
	Volume.fx_update();
}

//does the actual rotation of the layer
Volume.fx_onGetPixelR(double r, double d, double x, double y) {
	return r + KnobR;
}

System.onScriptUnloading() {
	delete SongTickerTimer;
	delete VolumeMap;
}